@@ -1,5 +1,10 @@
 import requests as rq
-import json, shutil, base64
+import json, shutil, base64, os, re
+from requests_toolbelt import MultipartEncoder
+
+
+help='''Данный модуль используется для работы с системой мониторига 2.0 (monitoring2), личным кабинетом 1-ofd.ru (prodRf) и GrayLog (grayLog).
+Зашифровать пароль: monitoring2.crypt(pas), где monitoring2 - название модуля monitoring2'''
 
 def encrypt(pas):
 	'''
@@ -19,7 +24,7 @@ def crypt(pas):
 	for x in pas:
 		if ord(x)+len(pas)>=128: crypto_pas+= chr(ord(x)+len(pas)-96)
 		else: crypto_pas+=chr(ord(x)+len(pas))
-	print('  crypto password: {}'.format(crypto_pas))
+	print('crypto password: "{}"'.format(crypto_pas))
 	return(crypto_pas)
 
 def responce(r):
@@ -34,9 +39,11 @@ def responce(r):
 	401: 'Не авторизован', 
 	403: 'Отказано в доступе', 
 	404: 'Не найдено', 
-	409: 'Конфликт', 
+	409: 'Конфликт',
+	415: 'Неподдерживаемый тип данных',
 	500: 'Ошибка',
-	503: 'Сервер перегружен'
+	503: 'Сервер перегружен',
+	504: 'Шлюз не отвечает слишком долго'
 	}
 	try: return answer[r.status_code]
 	except KeyError: return r.status_code
@@ -69,6 +76,7 @@ class monitoring2:
 	seek_by_inn(inn, rows = 50, page = 1) - Поиск организации по ИНН, ОГРН
 	seek_by_orgId(orgId) - Поиск организации по id
 	seek_by_contract(contract) - Поиск организации по номеру договора
+	get_all_kkt(org_id, page = 1, rows = 10000, sidx = 'fns_ecr_id', sord = 'desc') - Поиск всех ККТ организации по id организации
 	seek_kkm(kkm) - Поиск ККТ по id, РНМ, ФН
 	seek_kkm_by_id(id) - Поиск ККТ по id. Выгружается более полная информация
 	seek_transaction(transaction) - Поиск транзакции по id
@@ -79,25 +87,42 @@ class monitoring2:
 	ctrl(org_id ,ctrl_login) - Привязываание учётной записи ctrl к ЛК
 	user_by_mail(mail) - Поиск пользователя по почте
 	user_by_id(id) - Поиск пользователя по id
+	change_tenant(org_id, tenant_id) - Смена тенанта организации по id
 	'''
 
-	def __init__(self, login, password, url = 'http://taxplayer.ensyco.local', crpt = False):
+	def __init__(self, login, password, url = 'http://taxplayer.ensyco.local', crpt = False, echo = True):
 		self.URL = url
 		self.mon_login = login
 		self.__mon_password = password
 		if crpt: self.__mon_password = encrypt(pas = password)
 		self.log = []
+		self.echo = echo
+		self.ms = rq.Session()
 		self.login()
 
+	def __enter__(self):
+		return self
+
+	def __exit__(self, *arg):
+		self.logout()
+		del self.log
+		del self.mon_login
+		del self.__mon_password
+		del self.ms
+
 	def logger(self, text):
 		'''
 		Логгирование
 		'''
 		self.log.append(text)
-		self.log.append(self.r.status_code)
-		print(text, responce(self.r))
+		self.log.append(responce(self.r))
+		if self.echo: print(text, responce(self.r))
 		if self.r.status_code == 200:
-			return self.r.json()
+			try: 
+				return self.r.json()
+			except Exception as err:
+				print(err)
+				return self.r
 		else: 
 			return self.r.status_code		
 
@@ -105,7 +130,6 @@ class monitoring2:
 		'''
 		Создание сессии (логин)
 		'''
-		self.ms = rq.Session()
 		json = {"username":self.mon_login,"password":self.__mon_password}
 		self.r = self.ms.post('{}/api/login'.format(self.URL), json = json)
 		try:
@@ -122,13 +146,23 @@ class monitoring2:
 			return self.logger('Авторизация в мониторинге...')
 		except KeyError:
 			return self.logger('Логин или пароль не верный!!!')
+
+	def logout(self):
+		'''
+		Выход из мониторинга
+		'''
+		self.r = self.ms.get('{}/login'.format(self.URL))
+		return self.logger('Выход из мониторинга...')
 	
 	def seek_by_inn(self, inn, rows = 50, page = 1, sidx = 'orgId', sord = 'desc'):
 		'''
 		Поиск организации по ИНН, ОГРН
 		'''
-		form = {'rows' : rows, 'page' : page , 'sidx' : sidx, 'sord' : sord, 'orgTag' : inn}
-		self.r = self.ms.post('{}/api/organisations/filter'.format(self.URL), data=form)
+		old_header = self.ms.headers
+		self.ms.headers.update({'Content-Type': 'application/x-www-form-urlencoded'})
+		data={'orgTag':inn, 'page':page, 'rows':rows, 'sidx':'orgId'}
+		self.r = self.ms.post('{0}/api/organisations/filter'.format(self.URL), data=data)
+		self.ms.headers.update({'Content-Type': 'application/json'})
 		return self.logger('Поиск организации {}...'.format(inn))
 
 	def seek_by_orgId(self, orgId):
@@ -145,6 +179,17 @@ class monitoring2:
 		self.r = self.ms.get('{}/api/organisations/byContract?id={}'.format(self.URL, contract))
 		return self.logger('Поиск организации с договором {}...'.format(contract))
 
+	def get_all_kkt(self, org_id, page = 1, rows = 10000, sidx = 'fns_ecr_id', sord = 'desc'):
+		'''
+		Поиск всех ККТ организации по id организации
+		'''
+		old_header = self.ms.headers
+		self.ms.headers.update({'Content-Type': 'application/x-www-form-urlencoded'})
+		data={'orgId':org_id, 'page':page, 'rows':rows, 'sidx':'orgId'}
+		self.r = self.ms.post('{0}/api/organisations/{1}/kkms'.format(self.URL, org_id), data=data)
+		self.ms.headers.update({'Content-Type': 'application/json'})
+		return self.logger('Поиск всех ККТ организации id {}...'.format(org_id))
+
 	def seek_kkm(self, kkm):
 		'''
 		Поиск ККТ по id, РНМ, ФН
@@ -157,7 +202,7 @@ class monitoring2:
 		Поиск ККТ по id. Выгружается более полная информация
 		'''
 		self.r = self.ms.get('{}/api/kkms/{}'.format(self.URL, id))
-		return self.logger('Поиск ККТ id{}...'.format(id))
+		return self.logger('Поиск ККТ id {}...'.format(id))
 
 	def seek_transaction(self, transaction):
 		'''
@@ -170,9 +215,8 @@ class monitoring2:
 		'''
 		Планирование отчёта
 		'''
-		# form = {"clazz":clazz ,"distributionList":distributionList,"inns":inns,"kkmIdlePeriod":kkmIdlePeriod,"priority":priority,"scheduleDate":scheduleDate,"fromDate":fromDate,"toDate":toDate}
 		self.r = self.ms.post('{}/api/reports/generate'.format(self.URL), json=form)
-		return self.logger('Планирование отчёта {}...'.format(form))
+		return self.logger('Планирование отчёта ...')
 
 	def get_report_info(self, uuid):
 		'''
@@ -195,14 +239,15 @@ class monitoring2:
 		'''
 		Отмена ранее запланированного отчёта
 		'''
-		self.r = self.ms.get('{}/api/reports/{}/cancel'.format(self.URL, uuid))
+		self.r = self.ms.post('{}/api/reports/{}/cancel'.format(self.URL, uuid), json={})
 		return self.logger('Отмена отчёта {}...'.format(uuid))
 
 	def ctrl(self, org_id ,ctrl_login):
 		'''
 		Привязываание учётной записи ctrl к ЛК
 		'''
-		form = {"orgId" : int(org_id), 'serviceLogin' : ctrl_login}
+		form = {'orgId' : int(org_id), 'serviceLogin' : ctrl_login}
+		# self.ms.headers.update({'Content-Type': 'application/json'})
 		self.r = self.ms.post('{}/api/organisations/assign'.format(self.URL), json=form)
 		return self.logger('Контролимся к {}...'.format(org_id))
 
@@ -220,6 +265,14 @@ class monitoring2:
 		self.r = self.ms.get('{}/api/users/{}'.format(self.URL, id))
 		return self.logger('Поиск пользователя с id {}...'.format(id))
 
+	def change_tenant(self, org_id, tenant_id):
+		'''
+		Смена теннанта организации по id
+		'''
+		form = {"orgId":int(org_id),"tenantId":int(tennant_id)}
+		self.r = self.ms.post('{}/api/organisations/changetenant'.format(self.URL), json=form)
+		return self.logger('Меняем тенант организации {} на {}...'.format(org_id, tenant_id))
+
 class prodRf:
 
 	help = '''
@@ -227,6 +280,8 @@ class prodRf:
 	Инициализация prodRf(login , password , URL = 'https://api.1-ofd.ru', crpt = False), где crpt - зашифрованый пароль
 	login - Авторизация. Запускается автоматически при инициализации объекта, но можно запустить принудительно.
 	org_info - Получение информации о организации.
+	all_orgs_info - Получение информации о доступных пользователю организациях.
+	switch_org(org_id) - Переключение на организацию с org_id.
 	all_user - Получение списка пользователей.
 	create_user(email) - Создание нового пользователя с полными правами.
 	restore_password(email) - Восстановление пароля пользователя.
@@ -234,9 +289,32 @@ class prodRf:
 	all_kkt - Кассы организаций
 	api_version - Версия API
 	all_retail_places - Получение списка всех доступных торговых точек
+	retail_place_info(tt_id) - Получение информации о торговой точке по id
 	kkm_info(kkm_id) - Получение данных по ККТ с данным идентификатором
 	kkm_delete(kkm_id) - Удаление ККТ с данным идентификатором (не работает)
-	kkm_transactions(kkm_id, fiscalDriveNumber, **kwargs) - Получение списка операций по ККТ с данным идентификатором
+	kkm_transactions(kkm_id, fiscalDriveNumber, kwargs) - Получение списка операций по ККТ с данным идентификатором
+		kwargs = {'shiftNumber': '', 'fromDate': '', 'toDate': '', 'transactionsTypes': '', 'pageSize': '', 'page': 'fiscalDriveNumber', '':''}
+		shiftNumber - номер смены, необязательный
+		fromDate - начальная дата, необязательный  unix в мс
+		toDate - конечная дата, необязательный  unix в мс
+		transactionsTypes - тип операций, необязательный(TICKET, 
+			CLOSE_SHIFT, OPEN_SHIFT, FISCAL_REPORT, CLOSE_ARCHIVE, RECEIPT_CORRECTION, 
+			CURRENT_STATE_REPORT, FISCAL_REPORT_CORRECTION, BSO, BSO_CORRECTION), можно указывать через запятую
+		pageSize - размер страницы, необязательный
+		page - номер страницы, необязательный
+		fiscalDriveNumber - номер ФН, не обязательный
+	kkm_export_transactions(self, kkm_id, kwargs, file_name = False) - Экспорт операций по ККТ с данным идентификатором
+		kwargs = {'shiftNumber': '', 'fromDate': '', 'toDate': '', 'transactionsTypes': '', 'pageSize': '', 'page': 'fiscalDriveNumber', '':''}
+		shiftNumber - номер смены, необязательный
+		fromDate - начальная дата, необязательный  unix в мс
+		toDate - конечная дата, необязательный  unix в мс
+		transactionsTypes - тип операций, необязательный 
+			(TICKET, CLOSE_SHIFT, OPEN_SHIFT, FISCAL_REPORT, CLOSE_ARCHIVE, RECEIPT_CORRECTION, 
+			CURRENT_STATE_REPORT, FISCAL_REPORT_CORRECTION, BSO, BSO_CORRECTION), можно указывать через запятую
+		pageSize - размер страницы, необязательный
+		page - номер страницы, необязательный
+		fiscalDriveNumber - номер ФН, не обязательный
+		file_name = False - путь и имя файла, не обязательный
 	ticket(transactionId) - Просмотр чека по определенной транзакции
 	transaction(transactionId) - Просмотр чека по определенной транзакции
 	kkms_count - Получение количества ККТ
@@ -246,22 +324,38 @@ class prodRf:
 	process_fiscal_report(report_id) - Функция обработки отчёта о регистрации (для админки), требует логина в админку
 	set_tariff(kkmId, tariffId) - Установка тарифа по id tariffId для ККТ с id kkmId
 	activate_by_promo(kkmId, promo, agentCode = '' ) - Активация ТОЛЬКО ПРОМОтарифа
-	activate(self, kkmId, agentCode = '') - Активация тарифа (НЕ промотарифа)
+	activate(kkmId, agentCode = '') - Активация тарифа (НЕ промотарифа)
+	document_upload(name, file_name) - Выгрузка документа name - название документа в ЛК, file_name - файл, поддерживаемые форматы в self.file_extention_allowed
 	'''
 	
-	def __init__(self, login , password , URL = 'https://api.1-ofd.ru', crpt = False):
+	def __init__(self, login , password , URL = 'https://org.1-ofd.ru', crpt = False, echo = True):
 		self.log = []
 		self.lk_login = login
 		self.__lk_password = password
 		if crpt: self.__lk_password = encrypt(pas = password)
 		self.URL = URL
+		self.file_extention_allowed = {'pdf':'application/pdf',
+		'zip':'application/x-zip-compressed',
+		'7z':'application/octet-stream',
+		'rar':'application/octet-stream'}
+		self.echo = echo
 		self.ts = rq.Session()
 		self.login()
 
+	def __enter__(self):
+		return self
+
+	def __exit__(self, *arg):
+		self.logout()
+		del self.log
+		del self.lk_login
+		del self.__lk_password
+		del self.ts
+
 	def logger(self, text):
 		self.log.append(text)
 		self.log.append(responce(self.r))
-		print(text, responce(self.r))
+		if self.echo: print(text, responce(self.r))
 		if self.r.status_code == 200:
 			try: return self.r.json()
 			except json.decoder.JSONDecodeError: return self.r
@@ -277,10 +371,10 @@ class prodRf:
 		self.log.append(responce(self.r))
 		try:
 			self.ts.headers.update({'X-XSRF-TOKEN': self.r.cookies['PLAY_SESSION']})
-			return self.logger('Авторизация...') 
+			return self.logger('Авторизация в ЛК...') 
 		except Exception as err:
 			self.log.append(err)
-			return self.logger('Авторизация не удалась !!!!')
+			return self.logger('Авторизация не удалась!!!!')
 
 	def org_info(self):
 		'''
@@ -289,6 +383,22 @@ class prodRf:
 		self.url = '{}/api/organisation'.format(self.URL)
 		self.r = self.ts.get(self.url)
 		return self.logger('Получение информации о организации...')
+	
+	def all_orgs_info(self):
+		'''
+		Получение информации о доступных организациях.
+		'''
+		self.url = '{}/api/user/organisations'.format(self.URL)
+		self.r = self.ts.get(self.url)
+		return self.logger('Получение информации о доступных организациях...')
+
+	def switch_org(self, org_id):
+		'''
+		Переключение на организацию с org_id.
+		'''
+		self.url = '{}/api/user/organisations/{}'.format(self.URL, org_id)
+		self.r = self.ts.put(self.url)
+		return self.logger('Переключение на организацию с id {}...'.format(org_id))		
 
 	def all_user(self):
 		'''
@@ -323,7 +433,6 @@ class prodRf:
 		Обязательные входные данные: 
 		ИНН орг-ции(inn)*, почта пользователя(email)* (* - именованный)
 		'''
-		print(self.org_info()['inn'])
 		inn = self.org_info()['inn']
 		self.url = '{}/api/user/restore-password'.format(self.URL)
 		json = {"inn":inn,"email":email}
@@ -363,6 +472,14 @@ class prodRf:
 		self.r = self.ts.get(self.url)
 		return self.logger('Получение списка всех доступных торговых точек...')
 
+	def retail_place_info(self, tt_id):
+		'''
+		Получение информации о торговой точке с id
+		'''
+		self.url = '{}/api/kkms/filter-retail-place/{}'.format(self.URL, tt_id)
+		self.r = self.ts.post(self.url, json={})
+		return self.logger('Получение информации о торговой точке с id {}...'.format(tt_id))
+
 	def kkm_info(self, kkm_id):
 		'''
 		Получение данных по ККТ с данным идентификатором
@@ -371,6 +488,14 @@ class prodRf:
 		self.r = self.ts.get(self.url)
 		return self.logger('Получение данных по ККТ с id {}...'.format(kkm_id))
 
+	def kkm_history(self, kkm_id):
+		'''
+		Получение истории изменений по определенной ККТ с id
+		'''
+		self.url = '{}/api/kkms/{}/history'.format(self.URL, kkm_id)
+		self.r = self.ts.get(self.url)
+		return self.logger('Получение истории изменений по ККТ с id {}...'.format(kkm_id))		
+
 	def kkm_delete(self, kkm_id):
 		'''
 		Удаление ККТ с данным идентификатором (не работает)
@@ -379,18 +504,19 @@ class prodRf:
 		self.r = self.ts.delete(self.url)
 		return self.logger('Удаление ККТ с id {}...'.format(kkm_id))
 
-	def kkm_transactions(self, kkm_id, fiscalDriveNumber, **kwargs):
+	def kkm_transactions(self, kkm_id, kwargs):
 		'''
 		Получение списка операций по ККТ с данным идентификатором
-		kwargs = {'shiftNumber': '', 'fromDate': '', 'toDate': '', 'transactionsTypes': '', 'pageSize': '', 'page': ''}
+		kwargs = {'shiftNumber': '', 'fromDate': '', 'toDate': '', 'transactionsTypes': '', 'pageSize': '', 'page': 'fiscalDriveNumber', '':''}
 		shiftNumber - номер смены, необязательный
-		fromDate - начальная дата, необязательный  unix * 1000
-		toDate - конечная дата, необязательный  unix * 1000
+		fromDate - начальная дата, необязательный  unix в мс
+		toDate - конечная дата, необязательный  unix в мс
 		transactionsTypes - тип операций, необязательный 
 			(TICKET, CLOSE_SHIFT, OPEN_SHIFT, FISCAL_REPORT, CLOSE_ARCHIVE, RECEIPT_CORRECTION, 
 			CURRENT_STATE_REPORT, FISCAL_REPORT_CORRECTION, BSO, BSO_CORRECTION)
 		pageSize - размер страницы, необязательный
 		page - номер страницы, необязательный
+		fiscalDriveNumber - номер ФН, не обязательный
 		'''
 		url = '{}/api/kkms/{}/transactions?'.format(self.URL, kkm_id)
 		if kwargs.get('shiftNumber') and not kwargs.get('toDate'): url += 'shiftNumber={}&'.format(kwargs.get('shiftNumber'))
@@ -399,9 +525,51 @@ class prodRf:
 		if kwargs.get('transactionsTypes'): url += 'transactionsTypes={}&'.format(kwargs.get('transactionsTypes'))
 		if kwargs.get('pageSize'): url += 'pageSize={}&'.format(kwargs.get('pageSize'))
 		if kwargs.get('page'): url += 'page={}&'.format(kwargs.get('page'))
-		url += 'fiscalDriveNumber={}'.format(fiscalDriveNumber)
+		if kwargs.get('fiscalDriveNumber'): url += 'fiscalDriveNumber={}'.format(fiscalDriveNumber)
 		self.r = self.ts.get(url)
 		return self.logger('Получение списка операций по ККТ с id {}...'.format(kkm_id))
+
+	def kkm_export_transactions(self, kkm_id, kwargs, file_name = False):
+		'''
+		Получение списка операций по ККТ с данным идентификатором
+		kwargs = {'shiftNumber': '', 'fromDate': '', 'toDate': '', 'transactionsTypes': '', 'pageSize': '', 'page': 'fiscalDriveNumber', '':''}
+		shiftNumber - номер смены, необязательный
+		fromDate - начальная дата, необязательный  unix в мс
+		toDate - конечная дата, необязательный  unix в мс
+		transactionsTypes - тип операций, необязательный 
+			(TICKET, CLOSE_SHIFT, OPEN_SHIFT, FISCAL_REPORT, CLOSE_ARCHIVE, RECEIPT_CORRECTION, 
+			CURRENT_STATE_REPORT, FISCAL_REPORT_CORRECTION, BSO, BSO_CORRECTION)
+		pageSize - размер страницы, необязательный
+		page - номер страницы, необязательный
+		fiscalDriveNumber - номер ФН, не обязательный
+		file_name = False - путь и имя файла, не обязательный
+		'''
+		url = '{}/api/kkms/{}/export-transactions?'.format(self.URL, kkm_id)
+		if kwargs.get('shiftNumber') and not kwargs.get('toDate'): url += 'shiftNumber={}&'.format(kwargs.get('shiftNumber'))
+		if kwargs.get('fromDate'):  url += 'fromDate={}&'.format(kwargs.get('fromDate'))
+		if kwargs.get('toDate'):  url += 'toDate={}&'.format(kwargs.get('toDate'))
+		if kwargs.get('transactionsTypes'): url += 'transactionsTypes={}&'.format(kwargs.get('transactionsTypes'))
+		if kwargs.get('pageSize'): url += 'pageSize={}&'.format(kwargs.get('pageSize'))
+		if kwargs.get('page'): url += 'page={}&'.format(kwargs.get('page'))
+		if kwargs.get('fiscalDriveNumber'): url += 'fiscalDriveNumber={}'.format(fiscalDriveNumber)
+		self.r = self.ts.get(url, stream = True)
+		try:
+			file_name_site = re.findall('filename="(.+)"', self.r.headers['Content-Disposition'])[0]
+		except IndexError:
+			file_name_site = f'report-kkt-{kkm_id}.xlsx'
+		except KeyError:
+			self.r.status_code=504
+			return self.logger('Экспорт операций по ККТ с id {}...'.format(kkm_id))
+		if not file_name:
+			file_name = file_name_site
+		elif file_name.endswith('.xlsx') or file_name.endswith('.xls'):
+			file_name = file_name
+		else:
+			file_name = '{0}\\{1}'.format(file_name.rstrip('\\'), file_name_site)
+		with open(file_name, "wb") as file:
+			file.write(self.r.content)
+		return self.logger('Экспорт операций по ККТ с id {}...'.format(kkm_id))
+
 		
 	def ticket(self, transactionId):
 		'''
@@ -493,7 +661,30 @@ class prodRf:
 		url = '{}/api/billing/activate'.format(self.URL)
 		json={"kkmIds":int(kkmId),"agentCode": agentCode}
 		self.r = self.ts.post(url, json = json)
-		return self.logger('Активация тарифа (не промотарифа) для ККТ id {} с кодом Агента {}... '.format(kkmId, agentCode))		
+		return self.logger('Активация тарифа (не промотарифа) для ККТ id {} с кодом Агента {}... '.format(kkmId, agentCode))
+
+	def document_upload(self, name, file_name):
+		'''
+		Выгрузка документа, поддерживаемые форматы в self.file_extention_allowed
+		'''
+		file_extention = file_name.split('.')[-1]
+		if file_extention in self.file_extention_allowed.keys():
+			url = '{}/api/documents/upload'.format(self.URL)
+			json = {"filename": ('{}.{}'.format(name, file_extention))}
+			files = {"data":(file_name.split('\\')[-1], open(file_name, 'rb'), self.file_extention_allowed[file_extention])}
+			self.r = self.ts.post(url, data = json, files = files)
+			return self.logger('Загрузка файла {} ... '.format(file_name.split('\\')[-1]))
+		else:
+			return 'Формат *.{0} не поддерживается'.format(file_extention)
+
+	def get(self, kwargs):
+		return self.ts.get(**kwargs)
+
+	def post(self, kwargs):
+		return self.ts.post(**kwargs)
+
+
+	
 
 
 class grayLog:
@@ -506,23 +697,34 @@ class grayLog:
 
 	'''
 
-	def __init__(self, login , password , host = '10.1.102.24', crpt = False):
+	def __init__(self, login , password , host = '10.1.102.24', crpt = False, echo = True):
 		self.log = []
 		self.gl_login = login
 		self.__gl_password = password
 		if crpt: self.__gl_password = encrypt(pas = password)
 		self.host = host
 		self.URL = 'http://{}/'.format(host)
+		self.echo = echo
 		self.gs = rq.Session()
 		self.login()
 
+	def __enter__(self):
+		return self
+
+	def __exit__(self, *arg):
+		self.logout()
+		del self.log
+		del self.gl_login
+		del self.__gl_password
+		del self.gs
+
 	def logger(self, text):
 		'''
 		Логгирование
 		'''
 		self.log.append(text)
 		self.log.append(self.r.status_code)
-		print(text, responce(self.r))
+		if self.echo: print(text, responce(self.r))
 		if self.r.status_code == 200:
 			return self.r.json()
 		else: 
@@ -542,6 +744,12 @@ class grayLog:
 		except KeyError:
 			return self.logger('Авторизация в GrayLog не удалась...')
 
+	def logout(self):
+		url = 'http://{}/api/system/sessions'.format(self.host)
+		self.r = self.gs.get(url)
+		return self.logger('Выход из GrayLog...')
+		
+
 	def user(self):
 		url = 'http://{}/api/users/{}'.format(self.host, self.gl_login)
 		self.r = self.gs.get(url)
@@ -562,4 +770,5 @@ class grayLog:
 
 	def request(self, get_url):
 		url = self.URL + get_url
-		self.r = self.gs.get(url)
\ No newline at end of file
+		self.r = self.gs.get(url)
+
